package de.tud.kom.p2psim.impl.loadgen.config;

import java.util.HashSet;

import de.tud.kom.p2psim.impl.loadgen.distribution.AbstractDistribution;
import de.tudarmstadt.maki.simonstrator.api.util.XMLConfigurableConstructor;

/**
 * A class representing one request
 * 
 * @author Marc Schiller
 * @version 1.0, May 30, 2016
 */
public class Request {

	// Storage
	private RequestType requestType;

	private HashSet<Source> sources = new HashSet<>();

	private AbstractDistribution distribution;

	/**
	 * Constructor
	 */
	@XMLConfigurableConstructor({})
	public Request() {
	}

	/**
	 * Set the requestType for this request
	 * 
	 * @param requestType
	 */
	public void setRequestType(RequestType requestType) {
		this.requestType = requestType;
	}

	/**
	 * Add a source to this request
	 * 
	 * @param source
	 */
	public void setSource(Source source) {
		this.sources.add(source);
	}

	/**
	 * Set the distribution for this request
	 * 
	 * @param distribution
	 */
	public void setDistribution(AbstractDistribution distribution) {
		this.distribution = distribution;
	}

	/**
	 * Get the Distribution of this Request
	 * 
	 * @return
	 */
	public AbstractDistribution getDistribution() {
		return distribution;
	}

	/**
	 * Get the conditions for sources (those nodes who request data from other
	 * nodes) of this request
	 * 
	 * @return
	 */
	public HashSet<Source> getSources() {
		return this.sources;
	}

	/**
	 * Get the request that should be triggered from the Sources
	 * 
	 * @return
	 */
	public RequestType getRequestType() {
		return this.requestType;
	}

	/**
	 * Pretty print the Request
	 */
	public String toString() {
		String retString = "\tRequestType: " + this.requestType.toString();
		retString += "\tDistribution: " + this.distribution.toString() + "\n";

		for (Source src : this.sources) {
			retString += "\tSource: " + src.toString() + "\n";
		}

		return retString;
	}

}
