/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.resourcedistribution;

import de.tudarmstadt.maki.simonstrator.api.component.resourcedistribution.DistributableResource;
import de.tudarmstadt.maki.simonstrator.api.component.resourcedistribution.DistributableResource.RESOURCE_TYPE;

public class DefaultResourceContainer implements DistributableResource {
	
	private final RESOURCE_TYPE resourceType;
	private final String name;
	private final double maxValue;
	private double value;
	private boolean vital;
	
	public DefaultResourceContainer(RESOURCE_TYPE resourceType, String name, int maxValue, int initialValue, boolean vital) {
		this.resourceType = resourceType;
		this.name = name;
		this.maxValue = maxValue;
		this.value = initialValue;
		this.vital = vital;		
	}
	
	@Override
	public boolean isEmpty() {
		if(value <= 0) return true;
		
		return false;
	}

	@Override
	public RESOURCE_TYPE getResourceType() {
		return resourceType;
	}

	@Override
	public void setAmount(double amount) {
		if(amount > maxValue) 
			value = maxValue;
		else 
			value = amount;
	}

	@Override
	public void setPercentagedAmount(double percentage) {
		double percentageAmount = maxValue * percentage;
		this.setAmount(percentageAmount);
	}

	@Override
	public double getAmount() {
		return value;
	}

	@Override
	public double getMaxAmount() {
		return maxValue;
	}

	@Override
	public double fill(double amount) {
		double newAmount = value + amount;
		this.setAmount(newAmount);
		return value;
	}

	@Override
	public double extract(double amount) {
		if(amount > value) {
			setAmount(value - amount);
			return amount;
		}
		else {
			double rest = value;
			setAmount(0);
			return rest;
		}			
	}

}
