package de.tud.kom.p2psim.impl.resourcedistribution.factory;

import de.tud.kom.p2psim.api.common.SimHost;
import de.tud.kom.p2psim.api.resourcedistribution.ResourceModel;
import de.tud.kom.p2psim.impl.resourcedistribution.DefaultResourceModel;
import de.tud.kom.p2psim.impl.resourcedistribution.StatefulResourceConsumption;
import de.tudarmstadt.maki.simonstrator.api.Host;
import de.tudarmstadt.maki.simonstrator.api.Time;
import de.tudarmstadt.maki.simonstrator.api.component.ComponentNotAvailableException;
import de.tudarmstadt.maki.simonstrator.api.component.HostComponent;
import de.tudarmstadt.maki.simonstrator.api.component.HostComponentFactory;
import de.tudarmstadt.maki.simonstrator.api.component.resourcedistribution.DistributableResource.RESOURCE_TYPE;
import de.tudarmstadt.maki.simonstrator.api.component.resourcedistribution.ResourceConsumer;

/**
 * Provides a simple {@link ResourceModel} with a single vital resource
 * and using the {@link StatefulResourceConsumption} model.
 * All hosts start with the same amount of energy.
 *
 * @author Tim Feuerbach
 */
public class DummyResourceModelFactory implements HostComponentFactory {
    private int startAmount = 1000;
    private int maxAmount = 1000;
    private double consumptionInRoaming = 1;
    private double consumptionInHeading = 3;
    private long consumptionInterval = Time.SECOND;

    @Override
    public HostComponent createComponent(Host host) {
        if (startAmount > maxAmount) {
            throw new IllegalArgumentException("Start resource amount is larger than max resource amount, aborting");
        }

        SimHost simHost = (SimHost) host;

        try {
            ResourceConsumer consumer = host.getComponent(ResourceConsumer.class);
            StatefulResourceConsumption energyComponent = new StatefulResourceConsumption(consumer,
                    consumptionInRoaming, consumptionInHeading, consumptionInterval);
            ResourceModel model = new DefaultResourceModel(simHost);
            model.registerResourceType(RESOURCE_TYPE.ENERGY, "Energy", maxAmount, startAmount, true);
            model.registerComponent(energyComponent, RESOURCE_TYPE.ENERGY);

            return model;
        } catch (ComponentNotAvailableException e) {
            throw new IllegalStateException("Could not create resource model: host must provide ResoureConsumer " +
                    "component.");
        }
    }

    public void setStartAmount(int startAmount) {
        this.startAmount = startAmount;
    }

    public void setMaxAmount(int maxAmount) {
        this.maxAmount = maxAmount;
    }

    public void setConsumptionInRoaming(double consumptionInRoaming) {
        this.consumptionInRoaming = consumptionInRoaming;
    }

    public void setConsumptionInHeading(double consumptionInHeading) {
        this.consumptionInHeading = consumptionInHeading;
    }

    public void setConsumptionInterval(long consumptionInterval) {
        this.consumptionInterval = consumptionInterval;
    }
}
