package de.tud.kom.p2psim.impl.topology.movement.modularosm.transition;

import java.util.LinkedList;
import java.util.List;
import java.util.Random;

import de.tud.kom.p2psim.api.topology.Topology;
import de.tud.kom.p2psim.api.topology.movement.SimLocationActuator;
import de.tud.kom.p2psim.impl.topology.PositionVector;
import de.tud.kom.p2psim.impl.topology.movement.modularosm.attraction.TargetAssignmentAttractionPoint;
import de.tudarmstadt.maki.simonstrator.api.Binder;
import de.tudarmstadt.maki.simonstrator.api.Randoms;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.AttractionPoint;

/**
 * A variant of the {@link SocialTransitionStrategy} that is not social at all;
 * if the social transition fires, will select a random point on the map. This
 * point will very likely not be accessible, so when using
 * {@link de.tud.kom.p2psim.impl.topology.movement.local.RealWorldStreetsMovement}
 * , you should set
 * {@link de.tud.kom.p2psim.impl.topology.movement.local.RealWorldStreetsMovement#setIgnoreObstaclesLastWaypoint(boolean)
 * ignoreOBstaclesLastWaypoint} to true.
 */
public class RandomWaypointTransitionStrategy extends SocialTransitionStrategy {

	private PositionVector worldDimensions;

	private Random random;

	public RandomWaypointTransitionStrategy() {
		super();
		random = Randoms.getRandom(RandomWaypointTransitionStrategy.class);
	}

	@Override
	protected AttractionPoint getNextAttractionPoint(SimLocationActuator ms) {
		if (random.nextDouble() < getSocialFactor()) {
			return getRandomWaypoint();
		} else {
			List<AttractionPoint> aps = new LinkedList<>(getAllAttractionPoints());
			return aps.get(random.nextInt(aps.size()));
		}
	}

	private AttractionPoint getRandomWaypoint() {
		return new TargetAssignmentAttractionPoint(
				new PositionVector(randomDouble(0, getWorldDimensions().getX()),
						randomDouble(0, getWorldDimensions().getY())));
	}

	private double randomDouble(double min, double max) {
		return min + (max - min) * getRandom().nextDouble();
	}

	private PositionVector getWorldDimensions() {
		if (worldDimensions == null) {
			this.worldDimensions = Binder.getComponentOrNull(Topology.class)
					.getWorldDimensions();
		}
		return worldDimensions;
	}

	private Random getRandom() {
		return random;
	}
}
